@@ -1,13 +1,13 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	@(#) mhdb.pl 1.10 97/05/13 11:00:54 @(#)
+##	@(#) mhdb.pl 2.2 98/03/03 14:27:42
 ##  Author:
 ##      Earl Hood       ehood@medusa.acs.uci.edu
 ##  Description:
 ##      MHonArc library defining routines for outputing database.
 ##---------------------------------------------------------------------------##
 ##    MHonArc -- Internet mail-to-HTML converter
-##    Copyright (C) 1995-1997	Earl Hood, ehood@medusa.acs.uci.edu
+##    Copyright (C) 1995-1998	Earl Hood, ehood@medusa.acs.uci.edu
 ##
 ##    This program is free software; you can redistribute it and/or modify
 ##    it under the terms of the GNU General Public License as published by
@@ -21,7 +21,8 @@
 ##
 ##    You should have received a copy of the GNU General Public License
 ##    along with this program; if not, write to the Free Software
-##    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+##    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+##    02111-1307, USA
 ##---------------------------------------------------------------------------##
 
 ##---------------------------------------------------------------------------
@@ -53,23 +54,31 @@ sub output_db {
 	&print_assoc(DB, 'HeadHeads', *HeadHeads);
 	&print_assoc(DB, 'Icons', *Icons);
 	&print_assoc(DB, 'IndexNum', *IndexNum);
-	&print_assoc(DB, "main'MIMECharSetConverters",
-			 *main'MIMECharSetConverters);
-	&print_assoc(DB, "main'MIMEFilters", *main'MIMEFilters);
-	&print_assoc(DB, "main'MIMEFiltersArgs", *main'MIMEFiltersArgs);
+	&print_assoc(DB, "readmail'MIMECharSetConverters",
+			 *readmail'MIMECharSetConverters);
+	&print_assoc(DB, "readmail'MIMECharSetConvertersSrc",
+			 *readmail'MIMECharSetConvertersSrc);
+	&print_assoc(DB, "readmail'MIMEFilters",
+			 *readmail'MIMEFilters);
+	&print_assoc(DB, "readmail'MIMEFiltersSrc",
+			 *readmail'MIMEFiltersSrc);
+	&print_assoc(DB, "readmail'MIMEFiltersArgs",
+			 *readmail'MIMEFiltersArgs);
 	&print_assoc(DB, 'MsgId', *MsgId);
 	&print_assoc(DB, 'Refs', *Refs);
 	&print_assoc(DB, 'Subject', *Subject);
 	&print_assoc(DB, 'UDerivedFile', *UDerivedFile);
 	&print_assoc(DB, 'Zone', *Zone);
 
-	&print_array(DB, 'CharSetRequires', *CharSetRequires);
+	&print_array(DB, 'DateFields', *DateFields)
+	    unless $IsDefault{'DATEFIELDS'};
 	&print_array(DB, 'FieldOrder', *FieldOrder);
+	&print_array(DB, 'FromFields', *FromFields)
+	    unless $IsDefault{'FROMFIELDS'};
 	&print_array(DB, 'Months', *Months);
 	&print_array(DB, 'months', *months);
 	&print_array(DB, 'OtherIdxs', *OtherIdxs);
 	&print_array(DB, 'PerlINC', *PerlINC);
-	&print_array(DB, 'Requires', *Requires);
 	&print_array(DB, 'TListOrder', *TListOrder);
 	&print_array(DB, 'Weekdays', *Weekdays);
 	&print_array(DB, 'weekdays', *weekdays);
@@ -84,6 +93,10 @@ sub output_db {
 	&print_var(DB, 'ExpireTime', *ExpireTime);
 	&print_var(DB, 'FROM', *FROM);
 	&print_var(DB, 'GMTDateFmt', *GMTDateFmt);
+	&print_var(DB, 'GzipExe', *GzipExe);
+	&print_var(DB, 'GzipFiles', *GzipFiles);
+	&print_var(DB, 'GzipLinks', *GzipLinks);
+	&print_var(DB, 'HtmlExt', *HtmlExt);
 	&print_var(DB, 'IDXSIZE', *IDXSIZE);
 	&print_var(DB, 'LocalDateFmt', *LocalDateFmt);
 	&print_var(DB, 'MAILTOURL', *MAILTOURL);
@@ -95,22 +108,26 @@ sub output_db {
 	&print_var(DB, 'MsgGMTDateFmt', *MsgGMTDateFmt);
 	&print_var(DB, 'MSGHEAD', *MSGHEAD);
 	&print_var(DB, 'MsgLocalDateFmt', *MsgLocalDateFmt);
+	&print_var(DB, 'MsgPrefix', *MsgPrefix);
 	&print_var(DB, 'MULTIIDX', *MULTIIDX);
 	&print_var(DB, 'NOMAILTO', *NOMAILTO);
 	&print_var(DB, 'NONEWS', *NONEWS);
-	&print_var(DB, 'NOSORT', *NOSORT);
 	&print_var(DB, 'NOURL', *NOURL);
 	&print_var(DB, 'NumOfMsgs', *NumOfMsgs);
 	&print_var(DB, 'NumOfPages', *NumOfPages);
 	&print_var(DB, 'THREAD', *THREAD);
+	&print_var(DB, 'SubArtRxp', *SubArtRxp);
+	&print_var(DB, 'SubReplyRxp', *SubReplyRxp);
+	&print_var(DB, 'UsingLASTPG', *UsingLASTPG);
 
 	# Main index resources
+	&print_var(DB, 'AUTHSORT', *AUTHSORT);
+	&print_var(DB, 'NOSORT', *NOSORT);
+	&print_var(DB, 'REVSORT', *REVSORT);
+	&print_var(DB, 'SUBSORT', *SUBSORT);
 	if ($MAIN) {
-	    &print_var(DB, 'AUTHSORT', *AUTHSORT);
 	    &print_var(DB, 'IDXNAME', *IDXNAME);
 	    &print_var(DB, 'IDXPREFIX', *IDXPREFIX);
-	    &print_var(DB, 'REVSORT', *REVSORT);
-	    &print_var(DB, 'SUBSORT', *SUBSORT);
 	    &print_var(DB, 'TITLE', *TITLE);
 
 	    &print_var(DB, 'AUTHBEG', *AUTHBEG)
@@ -151,114 +168,110 @@ sub output_db {
 	if ($THREAD) {
 	    &print_var(DB, 'TIDXNAME', *TIDXNAME);
 	    &print_var(DB, 'TIDXPREFIX', *TIDXPREFIX);
-	    &print_var(DB, 'TLEVELS', *TLEVELS);
-	    &print_var(DB, 'TREVERSE', *TREVERSE);
 	    &print_var(DB, 'TTITLE', *TTITLE);
-
-	    &print_var(DB, 'TCONTBEG', *TCONTBEG)
-					unless $IsDefault{'TCONTBEG'};
-	    &print_var(DB, 'TCONTEND', *TCONTEND)
-					unless $IsDefault{'TCONTEND'};
-	    &print_var(DB, 'TFOOT', *TFOOT)
-					unless $IsDefault{'TFOOT'};
-	    &print_var(DB, 'THEAD', *THEAD)
-					unless $IsDefault{'THEAD'};
 	    &print_var(DB, 'TIDXLABEL', *TIDXLABEL)
-					unless $IsDefault{'TIDXLABEL'};
+				    unless $IsDefault{'TIDXLABEL'};
 	    &print_var(DB, 'TIDXPGBEG', *TIDXPGBEG)
-					unless $IsDefault{'TIDXPGBEG'};
+				    unless $IsDefault{'TIDXPGBEG'};
 	    &print_var(DB, 'TIDXPGEND', *TIDXPGEND)
-					unless $IsDefault{'TIDXPGEND'};
-	    &print_var(DB, 'TINDENTBEG', *TINDENTBEG)
-					unless $IsDefault{'TINDENTBEG'};
-	    &print_var(DB, 'TINDENTEND', *TINDENTEND)
-					unless $IsDefault{'TINDENTEND'};
-	    &print_var(DB, 'TLIEND', *TLIEND)
-					unless $IsDefault{'TLIEND'};
-	    &print_var(DB, 'TLINONE', *TLINONE)
-					unless $IsDefault{'TLINONE'};
-	    &print_var(DB, 'TLINONEEND', *TLINONEEND)
-					unless $IsDefault{'TLINONEEND'};
-	    &print_var(DB, 'TLITXT', *TLITXT)
-					unless $IsDefault{'TLITXT'};
+				    unless $IsDefault{'TIDXPGEND'};
 	    &print_var(DB, 'TNEXTPGLINK', *TNEXTPGLINK)
-					unless $IsDefault{'TNEXTPGLINK'};
+				    unless $IsDefault{'TNEXTPGLINK'};
 	    &print_var(DB, 'TNEXTPGLINKIA', *TNEXTPGLINKIA)
-					unless $IsDefault{'TNEXTPGLINKIA'};
+				    unless $IsDefault{'TNEXTPGLINKIA'};
 	    &print_var(DB, 'TPREVPGLINK', *TPREVPGLINK)
-					unless $IsDefault{'TPREVPGLINK'};
+				    unless $IsDefault{'TPREVPGLINK'};
 	    &print_var(DB, 'TPREVPGLINKIA', *TPREVPGLINKIA)
-					unless $IsDefault{'TPREVPGLINKIA'};
-	    &print_var(DB, 'TSINGLETXT', *TSINGLETXT)
-					unless $IsDefault{'TSINGLETXT'};
-	    &print_var(DB, 'TSUBJECTBEG', *TSUBJECTBEG)
-					unless $IsDefault{'TSUBJECTBEG'};
-	    &print_var(DB, 'TSUBJECTEND', *TSUBJECTEND)
-					unless $IsDefault{'TSUBJECTEND'};
-	    &print_var(DB, 'TSUBLISTBEG', *TSUBLISTBEG)
-					unless $IsDefault{'TSUBLISTBEG'};
-	    &print_var(DB, 'TSUBLISTEND', *TSUBLISTEND)
-					unless $IsDefault{'TSUBLISTEND'};
-	    &print_var(DB, 'TTOPBEG', *TTOPBEG)
-					unless $IsDefault{'TTOPBEG'};
-	    &print_var(DB, 'TTOPEND', *TTOPEND)
-					unless $IsDefault{'TTOPEND'};
+				    unless $IsDefault{'TPREVPGLINKIA'};
 	}
+	&print_var(DB, 'TLEVELS', *TLEVELS);
+	&print_var(DB, 'TNOSORT', *TNOSORT);
+	&print_var(DB, 'TREVERSE', *TREVERSE);
+	&print_var(DB, 'TSUBSORT', *TSUBSORT);
+	&print_var(DB, 'TCONTBEG', *TCONTBEG) unless $IsDefault{'TCONTBEG'};
+	&print_var(DB, 'TCONTEND', *TCONTEND) unless $IsDefault{'TCONTEND'};
+	&print_var(DB, 'TFOOT', *TFOOT) unless $IsDefault{'TFOOT'};
+	&print_var(DB, 'THEAD', *THEAD) unless $IsDefault{'THEAD'};
+	&print_var(DB, 'TINDENTBEG', *TINDENTBEG)
+				    unless $IsDefault{'TINDENTBEG'};
+	&print_var(DB, 'TINDENTEND', *TINDENTEND)
+				    unless $IsDefault{'TINDENTEND'};
+	&print_var(DB, 'TLIEND', *TLIEND) unless $IsDefault{'TLIEND'};
+	&print_var(DB, 'TLINONE', *TLINONE) unless $IsDefault{'TLINONE'};
+	&print_var(DB, 'TLINONEEND', *TLINONEEND)
+				    unless $IsDefault{'TLINONEEND'};
+	&print_var(DB, 'TLITXT', *TLITXT) unless $IsDefault{'TLITXT'};
+	&print_var(DB, 'TSINGLETXT', *TSINGLETXT)
+				    unless $IsDefault{'TSINGLETXT'};
+	&print_var(DB, 'TSUBJECTBEG', *TSUBJECTBEG)
+				    unless $IsDefault{'TSUBJECTBEG'};
+	&print_var(DB, 'TSUBJECTEND', *TSUBJECTEND)
+				    unless $IsDefault{'TSUBJECTEND'};
+	&print_var(DB, 'TSUBLISTBEG', *TSUBLISTBEG)
+				    unless $IsDefault{'TSUBLISTBEG'};
+	&print_var(DB, 'TSUBLISTEND', *TSUBLISTEND)
+				    unless $IsDefault{'TSUBLISTEND'};
+	&print_var(DB, 'TTOPBEG', *TTOPBEG) unless $IsDefault{'TTOPBEG'};
+	&print_var(DB, 'TTOPEND', *TTOPEND) unless $IsDefault{'TTOPEND'};
 
-	&print_var(DB, 'BOTLINKS', *BOTLINKS)
-	    				unless $IsDefault{'BOTLINKS'};
-	&print_var(DB, 'FIELDSBEG', *FIELDSBEG)
-	    				unless $IsDefault{'FIELDSBEG'};
-	&print_var(DB, 'FIELDSEND', *FIELDSEND)
-					unless $IsDefault{'FIELDSEND'};
-	&print_var(DB, 'FLDBEG', *FLDBEG)
-	    				unless $IsDefault{'FLDBEG'};
-	&print_var(DB, 'FLDEND', *FLDEND)
-	    				unless $IsDefault{'FLDEND'};
+	## Other resources
+	&print_var(DB, 'BOTLINKS', *BOTLINKS) unless $IsDefault{'BOTLINKS'};
+	&print_var(DB, 'FIELDSBEG', *FIELDSBEG) unless $IsDefault{'FIELDSBEG'};
+	&print_var(DB, 'FIELDSEND', *FIELDSEND) unless $IsDefault{'FIELDSEND'};
+	&print_var(DB, 'FLDBEG', *FLDBEG) unless $IsDefault{'FLDBEG'};
+	&print_var(DB, 'FLDEND', *FLDEND) unless $IsDefault{'FLDEND'};
+	&print_var(DB, 'FOLUPBEGIN', *FOLUPBEGIN)
+	    				unless $IsDefault{'FOLUPBEGIN'};
+	&print_var(DB, 'FOLUPEND', *FOLUPEND) unless $IsDefault{'FOLUPEND'};
+	&print_var(DB, 'FOLUPLITXT', *FOLUPLITXT)
+	    				unless $IsDefault{'FOLUPLITXT'};
 	&print_var(DB, 'HEADBODYSEP ', *HEADBODYSEP)
 	    				unless $IsDefault{'HEADBODYSEP'};
-	&print_var(DB, 'LABELBEG', *LABELBEG)
-	    				unless $IsDefault{'LABELBEG'};
-	&print_var(DB, 'LABELEND', *LABELEND)
-	    				unless $IsDefault{'LABELEND'};
-	&print_var(DB, 'MSGPGBEG', *MSGPGBEG)
-	    				unless $IsDefault{'MSGPGBEG'};
-	&print_var(DB, 'MSGPGEND', *MSGPGEND)
-	    				unless $IsDefault{'MSGPGEND'};
+	&print_var(DB, 'LABELBEG', *LABELBEG) unless $IsDefault{'LABELBEG'};
+	&print_var(DB, 'LABELEND', *LABELEND) unless $IsDefault{'LABELEND'};
+	&print_var(DB, 'MSGBODYEND', *MSGBODYEND)
+	    				unless $IsDefault{'MSGBODYEND'};
+	&print_var(DB, 'MSGIDLINK', *MSGIDLINK) unless $IsDefault{'MSGIDLINK'};
+	&print_var(DB, 'MSGPGBEG', *MSGPGBEG) unless $IsDefault{'MSGPGBEG'};
+	&print_var(DB, 'MSGPGEND', *MSGPGEND) unless $IsDefault{'MSGPGEND'};
 	&print_var(DB, 'NEXTBUTTON', *NEXTBUTTON)
 	    				unless $IsDefault{'NEXTBUTTON'};
 	&print_var(DB, 'NEXTBUTTONIA', *NEXTBUTTONIA)
 	    				unless $IsDefault{'NEXTBUTTONIA'};
-	&print_var(DB, 'NEXTLINK', *NEXTLINK)
-	    				unless $IsDefault{'NEXTLINK'};
+	&print_var(DB, 'NEXTLINK', *NEXTLINK) unless $IsDefault{'NEXTLINK'};
 	&print_var(DB, 'NEXTLINKIA', *NEXTLINKIA)
 	    				unless $IsDefault{'NEXTLINKIA'};
 	&print_var(DB, 'PREVBUTTON', *PREVBUTTON)
 	    				unless $IsDefault{'PREVBUTTON'};
 	&print_var(DB, 'PREVBUTTONIA', *PREVBUTTONIA)
 	    				unless $IsDefault{'PREVBUTTONIA'};
-	&print_var(DB, 'PREVLINK', *PREVLINK)
-	    				unless $IsDefault{'PREVLINK'};
+	&print_var(DB, 'PREVLINK', *PREVLINK) unless $IsDefault{'PREVLINK'};
 	&print_var(DB, 'PREVLINKIA', *PREVLINKIA)
 	    				unless $IsDefault{'PREVLINKIA'};
+	&print_var(DB, 'REFSBEGIN', *REFSBEGIN) unless $IsDefault{'REFSBEGIN'};
+	&print_var(DB, 'REFSEND', *REFSEND) unless $IsDefault{'REFSEND'};
+	&print_var(DB, 'REFSLITXT', *REFSLITXT) unless $IsDefault{'REFSLITXT'};
+	&print_var(DB, 'SUBJECTHEADER ', *SUBJECTHEADER)
+	    				unless $IsDefault{'SUBJECTHEADER'};
 	&print_var(DB, 'TNEXTBUTTON', *TNEXTBUTTON)
 	    				unless $IsDefault{'TNEXTBUTTON'};
 	&print_var(DB, 'TNEXTBUTTONIA', *TNEXTBUTTONIA)
 	    				unless $IsDefault{'TNEXTBUTTONIA'};
-	&print_var(DB, 'TNEXTLINK', *TNEXTLINK)
-	    				unless $IsDefault{'TNEXTLINK'};
+	&print_var(DB, 'TNEXTLINK', *TNEXTLINK) unless $IsDefault{'TNEXTLINK'};
 	&print_var(DB, 'TNEXTLINKIA', *TNEXTLINKIA)
 	    				unless $IsDefault{'TNEXTLINKIA'};
-	&print_var(DB, 'TOPLINKS', *TOPLINKS)
-	    				unless $IsDefault{'TOPLINKS'};
+	&print_var(DB, 'TOPLINKS', *TOPLINKS) unless $IsDefault{'TOPLINKS'};
 	&print_var(DB, 'TPREVBUTTON', *TPREVBUTTON)
 	    				unless $IsDefault{'TPREVBUTTON'};
 	&print_var(DB, 'TPREVBUTTONIA', *TPREVBUTTONIA)
 	    				unless $IsDefault{'TPREVBUTTONIA'};
-	&print_var(DB, 'TPREVLINK', *TPREVLINK)
-	    				unless $IsDefault{'TPREVLINK'};
+	&print_var(DB, 'TPREVLINK', *TPREVLINK) unless $IsDefault{'TPREVLINK'};
 	&print_var(DB, 'TPREVLINKIA', *TPREVLINKIA)
 	    				unless $IsDefault{'TPREVLINKIA'};
+	&print_var(DB, 'TSLICEBEG', *TSLICEBEG) unless $IsDefault{'TSLICEBEG'};
+	&print_var(DB, 'TSLICEEND', *TSLICEEND) unless $IsDefault{'TSLICEEND'};
+	&print_var(DB, 'TSliceNBefore', *TSliceNBefore);
+	&print_var(DB, 'TSliceNAfter', *TSliceNAfter);
 	&print_var(DB, 'UMASK', *UMASK);
 
 	print DB "1;\n";	# for require
