@@ -1,13 +1,13 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	@(#) mhidxrc.pl 1.5 97/05/13 11:24:35 @(#)
+##	@(#) mhidxrc.pl 2.2 98/03/03 14:29:10
 ##  Author:
 ##      Earl Hood       ehood@medusa.acs.uci.edu
 ##  Description:
 ##      MHonArc library defining values for various index resources
 ##---------------------------------------------------------------------------##
 ##    MHonArc -- Internet mail-to-HTML converter
-##    Copyright (C) 1996,1997	Earl Hood, ehood@medusa.acs.uci.edu
+##    Copyright (C) 1996-1998	Earl Hood, ehood@medusa.acs.uci.edu
 ##
 ##    This program is free software; you can redistribute it and/or modify
 ##    it under the terms of the GNU General Public License as published by
@@ -21,9 +21,12 @@
 ##
 ##    You should have received a copy of the GNU General Public License
 ##    along with this program; if not, write to the Free Software
-##    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+##    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+##    02111-1307, USA
 ##---------------------------------------------------------------------------##
 
+package mhonarc;
+
 ##-----------------##
 ## Index resources ##
 ##-----------------##
@@ -32,7 +35,8 @@ $IdxTypeStr = $NOSORT ? 'Message' :
 		        $SUBSORT ? 'Subject' :
 			$AUTHSORT ? 'Author' :
 			'Date';
-if ($MAIN) {
+## MAIN index resources
+## if ($MAIN) {
 
     ##	Label for main index
     unless ($IDXLABEL) {
@@ -106,9 +110,11 @@ EndOfStr
     unless ($SUBJECTEND) {
 	$SUBJECTEND = ''; $IsDefault{'SUBJECTEND'} = 1;
     }
-}
 
-if ($THREAD) {
+## }
+
+## THREAD index resources
+## if ($THREAD) {
 
     ##	Label for thread index
     unless ($TIDXLABEL) {
@@ -246,12 +252,30 @@ EndOfStr
 
     $DoMissingMsgs = $TLINONE =~ /\S/;
 
+## }
+
+unless ($TSLICEBEG) {
+    $TSLICEBEG = "<BLOCKQUOTE><UL>\n";
+    $IsDefault{'TSLICEBEG'} = 1;
+}
+unless ($TSLICEEND) {
+    $TSLICEEND = "</UL></BLOCKQUOTE>\n";
+    $IsDefault{'TSLICEEND'} = 1;
 }
 
 ##-------------------##
 ## Message resources ##
 ##-------------------##
 
+unless (@DateFields) {
+    @DateFields = ('received', 'date');
+    $IsDefault{'DATEFIELDS'} = 1;
+}
+unless (@FromFields) {
+    @FromFields = ('from', 'reply-to', 'apparently-from');
+    $IsDefault{'FROMFIELDS'} = 1;
+}
+
 ## Beginning of message page
 unless ($MSGPGBEG) {
     $MSGPGBEG =<<'EndOfStr';
@@ -284,6 +308,12 @@ unless ($HEADBODYSEP) {
     $IsDefault{'HEADBODYSEP'} = 1;
 }
 
+## Separator between end of message data and rest of page
+unless ($MSGBODYEND) {
+    $MSGBODYEND = "<HR>\n";
+    $IsDefault{'MSGBODYEND'} = 1;
+}
+
 ##---------------------------------##
 ## Mail header formating resources ##
 ##---------------------------------##
@@ -387,7 +417,7 @@ if (!$TOPLINKS) {
 
 ## Bottom links in message
 if (!$BOTLINKS) {
-    $BOTLINKS =  "<HR>\n<UL>\n";
+    $BOTLINKS =  "<UL>\n";
     $BOTLINKS .= '$PREVLINK$$NEXTLINK$'  if $MAIN;
     $BOTLINKS .= '$TPREVLINK$$TNEXTLINK$'  if $THREAD;
     if ($MAIN || $THREAD) {
@@ -401,6 +431,49 @@ if (!$BOTLINKS) {
     $IsDefault{'BOTLINKS'} = 1;
 }
 
+## Follow-up and References resources
+unless ($FOLUPBEGIN) {
+    $FOLUPBEGIN =<<'EndOfVar';
+<UL><LI><STRONG>Follow-Ups</STRONG>:
+<UL>
+EndOfVar
+    $IsDefault{'FOLUPBEGIN'} = 1;
+}
+unless ($FOLUPLITXT) {
+    $FOLUPLITXT =<<'EndOfVar';
+<LI><STRONG>$SUBJECT$</STRONG>
+<UL><LI><EM>From:</EM> $FROM$</LI></UL></LI>
+EndOfVar
+    $IsDefault{'FOLUPLITXT'} = 1;
+}
+unless ($FOLUPEND) {
+    $FOLUPEND =<<'EndOfVar';
+</UL></LI></UL>
+EndOfVar
+    $IsDefault{'FOLUPEND'} = 1;
+}
+
+unless ($REFSBEGIN) {
+    $REFSBEGIN =<<'EndOfVar';
+<UL><LI><STRONG>References</STRONG>:
+<UL>
+EndOfVar
+    $IsDefault{'REFSBEGIN'} = 1;
+}
+unless ($REFSLITXT) {
+    $REFSLITXT =<<'EndOfVar';
+<LI><STRONG>$SUBJECT$</STRONG>
+<UL><LI><EM>From:</EM> $FROM$</LI></UL></LI>
+EndOfVar
+    $IsDefault{'REFSLITXT'} = 1;
+}
+unless ($REFSEND) {
+    $REFSEND =<<'EndOfVar';
+</UL></LI></UL>
+EndOfVar
+    $IsDefault{'REFSEND'} = 1;
+}
+
 ##--------------------------------------------##
 ## Next/previous main/thread index page links ##
 ##--------------------------------------------##
@@ -423,6 +496,9 @@ $TNEXTPGLINKIA = '[Next Page]',
 $TPREVPGLINKIA = '[Prev Page]',
     $IsDefault{'TPREVPGLINKIA'} = 1	unless $TPREVPGLINKIA;
 
+$MSGIDLINK = '<A $A_HREF$>$MSGID$</A>',
+     $IsDefault{'MSGIDLINK'} = 1	unless $MSGIDLINK;
+
 ##	Set unknown icon
 $Icons{'unknown'} = $Icons{'text/plain'}  unless $Icons{'unknown'};
 
