@@ -1,13 +1,13 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##	@(#) mhinit.pl 1.18 97/05/13 11:25:03 @(#)
+##	@(#) mhinit.pl 2.2 98/03/03 14:30:12
 ##  Author:
 ##      Earl Hood       ehood@medusa.acs.uci.edu
 ##  Description:
 ##      Initialization stuff for MHonArc.
 ##---------------------------------------------------------------------------##
 ##    MHonArc -- Internet mail-to-HTML converter
-##    Copyright (C) 1995-1997	Earl Hood, ehood@medusa.acs.uci.edu
+##    Copyright (C) 1995-1998	Earl Hood, ehood@medusa.acs.uci.edu
 ##
 ##    This program is free software; you can redistribute it and/or modify
 ##    it under the terms of the GNU General Public License as published by
@@ -21,9 +21,12 @@
 ##
 ##    You should have received a copy of the GNU General Public License
 ##    along with this program; if not, write to the Free Software
-##    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+##    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+##    02111-1307, USA
 ##---------------------------------------------------------------------------##
 
+package mhonarc;
+
 ##---------------------------------------------------------------------------##
 
 ##	The %Zone array should be augmented to contain all timezone
@@ -134,87 +137,88 @@ $DoMissingMsgs	=  0;	# Flag is missing messages should be noted in index
 
 ##	Some miscellaneous variables
 
-%IsDefault	= ();		# Flags if certain resources are the default
+%IsDefault	= ();	# Flags if certain resources are the default
 
-$bs 		= "\b";		# Used as a separator
+$bs 		= "\b";	# Used as a separator
 $Url 		= '(http://|https://|ftp://|afs://|wais://|telnet://|' .
 		   'gopher://|news:|nntp:|mid:|cid:|mailto:|prospero:)';
-$MLCP		= 0;		# Main index contains included files flag
-$SLOW		= 0;		# Save memory flag
-$NumOfPages	= 0;		# Number of index pages
-$IdxMinPg	= -1;		# Starting page of index for updating
-$TIdxMinPg	= -1;		# Starting page of thread index for updating
-$DBPathName	= '';		# Full pathname of database file
-$DefRcName	= ($MSDOS ? "mhonarc.rc" : ".mhonarc.rc");
 
-##	Set default filter libraries
-
-@Requires = (
-
-    "mhexternal.pl",
-    "mhtxthtml.pl",
-    "mhtxtplain.pl",
-    "mhtxtsetext.pl",
-
-);
-
-@CharSetRequires = (
-
-    "iso8859.pl",
-
-);
+$MLCP		= 0;	# Main index contains included files flag
+$SLOW		= 0;	# Save memory flag
+$NumOfPages	= 0;	# Number of index pages
+$IdxMinPg	= -1;	# Starting page of index for updating
+$TIdxMinPg	= -1;	# Starting page of thread index for updating
+$DBPathName	= '';	# Full pathname of database file
 
 ##----------------------------------------------------------------------
 ##	BEGIN readmail.pl variable settings
 ##----------------------------------------------------------------------
 ##	Default filters
 ##
-%main'MIMEFilters = (
-
+%readmail'MIMEFilters = (
     # Content-type			Filter
     #-------------------------------------------------------------------
     "application/x-patch",		"m2h_text_plain'filter",
     "message/partial",   		"m2h_text_plain'filter",
+    "text/enriched",    		"m2h_text_enriched'filter",
     "text/html",			"m2h_text_html'filter",
     "text/plain",			"m2h_text_plain'filter",
-    "text/richtext",    		"m2h_text_plain'filter",
+    "text/richtext",    		"m2h_text_enriched'filter",
     "text/setext",			"m2h_text_setext'filter",
-    "text/tab-separated-values",	"m2h_text_plain'filter",
+    "text/tab-separated-values",	"m2h_text_tsv'filter",
     "text/x-html",			"m2h_text_html'filter",
     "text/x-setext",    		"m2h_text_setext'filter",
 
     "application/*",		 	"m2h_external'filter",
     "audio/*",				"m2h_external'filter",
+    "chemical/*",  			"m2h_external'filter",
     "image/*",  			"m2h_external'filter",
+    "model/*",  			"m2h_external'filter",
     "text/*",   			"m2h_text_plain'filter",
     "video/*",  			"m2h_external'filter",
+);
+%readmail'MIMEFiltersSrc = (
+    # Content-type			Filter
+    #-------------------------------------------------------------------
+    "application/x-patch",		"mhtxtplain.pl",
+    "message/partial",   		"mhtxtplain.pl",
+    "text/enriched",    		"mhtxtenrich.pl",
+    "text/html",			"mhtxthtml.pl",
+    "text/plain",			"mhtxtplain.pl",
+    "text/richtext",    		"mhtxtenrich.pl",
+    "text/setext",			"mhtxtsetext.pl",
+    "text/tab-separated-values",	"mhtxttsv.pl",
+    "text/x-html",			"mhtxthtml.pl",
+    "text/x-setext",    		"mhtxtsetext.pl",
 
-    # Nead to add to list if more base types are registered.
-
+    "application/*",		 	"mhexternal.pl",
+    "audio/*",				"mhexternal.pl",
+    "chemical/*",  			"mhexternal.pl",
+    "image/*",  			"mhexternal.pl",
+    "model/*",  			"mhexternal.pl",
+    "text/*",   			"mhtxtplain.pl",
+    "video/*",  			"mhexternal.pl",
 );
 
 ##  Default filter arguments
 ##
-%main'MIMEFiltersArgs = (
-
+%readmail'MIMEFiltersArgs = (
     # Content-type			Arguments
     #-------------------------------------------------------------------
     "image/gif",			"inline",
     "image/jpeg",			"inline",
     "image/x-xbitmap", 	 		"inline",
     "image/x-xbm",			"inline",
-
 );
 
 ##  Charset filters
 ##
-%main'MIMECharSetConverters = (
-
+%readmail'MIMECharSetConverters = (
     # Character set			Converter Function
     #-------------------------------------------------------------------
-    "plain",     			"main'convert_line",
-    "us-ascii",   			"main'convert_line",
-    "iso-8859-1",   			"iso_8859'str2sgml",
+    "plain",     			"mhonarc'htmlize",
+    "us-ascii",   			"mhonarc'htmlize",
+    "iso-8859-1",   			"mhonarc'htmlize",
     "iso-8859-2",   			"iso_8859'str2sgml",
     "iso-8859-3",   			"iso_8859'str2sgml",
     "iso-8859-4",   			"iso_8859'str2sgml",
@@ -224,16 +228,32 @@ $DefRcName	= ($MSDOS ? "mhonarc.rc" : ".mhonarc.rc");
     "iso-8859-8",   			"iso_8859'str2sgml",
     "iso-8859-9",   			"iso_8859'str2sgml",
     "iso-8859-10",   			"iso_8859'str2sgml",
-
     "default",     			"-ignore-",
-
 );
+%readmail'MIMECharSetConvertersSrc = (
+    # Character set			Converter Function
+    #-------------------------------------------------------------------
+    "plain",     			undef,
+    "us-ascii",   			undef,
+    "iso-8859-1",   			undef,
+    "iso-8859-2",   			"iso8859.pl",
+    "iso-8859-3",   			"iso8859.pl",
+    "iso-8859-4",   			"iso8859.pl",
+    "iso-8859-5",   			"iso8859.pl",
+    "iso-8859-6",   			"iso8859.pl",
+    "iso-8859-7",   			"iso8859.pl",
+    "iso-8859-8",   			"iso8859.pl",
+    "iso-8859-9",   			"iso8859.pl",
+    "iso-8859-10",   			"iso8859.pl",
+    "default",     			undef,
+);
+
 ##------------------------------------------------------------------------
 ##	END readmail.pl variable settings
 ##------------------------------------------------------------------------
 
 ##  Variable to hold function for converting message header text.
-$MHeadCnvFunc	= "convert_line";
+$MHeadCnvFunc	= "mhonarc'htmlize";
 
 ##  Regexp for variable detection
 $VarExp = '\$([^\$]*)\$';
@@ -243,35 +263,42 @@ $AddrExp = q%[^()<>@,;:\/\s"'&|]+@[^()<>@,;:\/\s"'&|]+%;
 
 ##	Grab environment variable settings
 ##
-$DBFILE    = $ENV{'M2H_DBFILE'}     || ($MSDOS? "mhonarc.db": ".mhonarc.db");
+$DBFILE    = $ENV{'M2H_DBFILE'}     || 
+	     (($MSDOS || $VMS) ? "mhonarc.db": ".mhonarc.db");
 $DOCURL    = $ENV{'M2H_DOCURL'}     ||
 	     'http://www.oac.uci.edu/indiv/ehood/mhonarc.html';
 $FOOTER    = $ENV{'M2H_FOOTER'}     || "";
 $HEADER    = $ENV{'M2H_HEADER'}     || "";
-$IDXNAME   = $ENV{'M2H_IDXFNAME'}   || "maillist.html";
+$IDXNAME   = "";	# Set in get_cli_opts()
 $IDXPREFIX = $ENV{'M2H_IDXPREFIX'}  || "mail";
 $TIDXPREFIX= $ENV{'M2H_TIDXPREFIX'} || "thrd";
-$IDXSIZE   = $ENV{'M2H_IDXSIZE'}    || "";
-$TIDXNAME  = $ENV{'M2H_TIDXFNAME'}  || "threads.html";
+$IDXSIZE   = $ENV{'M2H_IDXSIZE'}    || 0;
+$TIDXNAME  = "";	# Set in get_cli_opts()
 $OUTDIR    = $ENV{'M2H_OUTDIR'}     || $CURDIR;
 $FMTFILE   = $ENV{'M2H_RCFILE'}     || "";
 $TTITLE    = $ENV{'M2H_TTITLE'}     || "Mail Thread Index";
 $TITLE     = $ENV{'M2H_TITLE'}      || "Mail Index";
 $MAILTOURL = $ENV{'M2H_MAILTOURL'}  || "";
 $FROM      = $ENV{'M2H_MSGSEP'}     || '^From ';
-$LOCKFILE  = $ENV{'M2H_LOCKFILE'}   || ($MSDOS? "mhonarc.lck": ".mhonarc.lck");
+$LOCKFILE  = $ENV{'M2H_LOCKFILE'}   ||
+	     (($MSDOS || $VMS) ? "mhonarc.lck": ".mhonarc.lck");
 $LOCKTRIES = $ENV{'M2H_LOCKTRIES'}  || 10;
 $LOCKDELAY = $ENV{'M2H_LOCKDELAY'}  || 3;
-$MAXSIZE   = $ENV{'M2H_MAXSIZE'}    || "";
+$MAXSIZE   = $ENV{'M2H_MAXSIZE'}    || 0;
 $TLEVELS   = $ENV{'M2H_TLEVELS'}    || 3;
 $MHPATTERN = $ENV{'M2H_MHPATTERN'}  || '^\d+$';
 $DefRcFile = $ENV{'M2H_DEFRCFILE'}  || '';
+$HtmlExt   = $ENV{'M2H_HTMLEXT'}    || "html";
+$MsgPrefix = $ENV{'M2H_MSGPREFIX'}  || "msg";
+$DefRcName = $ENV{'M2H_DEFRCNAME'}  ||
+	     (($MSDOS || $VMS) ? "mhonarc.rc": ".mhonarc.rc");
+$GzipExe   = $ENV{'M2H_GZIPEXE'}    || 'gzip';
 
-$GMTDateFmt	= $ENV{'M2H_GMTDATEFMT'}   	|| '';
-$LocalDateFmt	= $ENV{'M2H_LOCALDATEFMT'}	|| '';
-$ExpireDate	= $ENV{'M2H_EXPIREDATE'}	|| '';
+$GMTDateFmt	= $ENV{'M2H_GMTDATEFMT'}   || '';
+$LocalDateFmt	= $ENV{'M2H_LOCALDATEFMT'} || '';
+$ExpireDate	= $ENV{'M2H_EXPIREDATE'}   || '';
     $ExpireDateTime = 0;
-$ExpireTime	= $ENV{'M2H_EXPIREAGE'} 	|| 0;
+$ExpireTime	= $ENV{'M2H_EXPIREAGE'}    || 0;
 
 $MsgGMTDateFmt	= $ENV{'M2H_MSGGMTDATEFMT'}   	|| '';
 $MsgLocalDateFmt= $ENV{'M2H_MSGLOCALDATEFMT'}	|| '';
@@ -289,7 +316,11 @@ $REVSORT     = defined($ENV{'M2H_REVSORT'}) ? $ENV{'M2H_REVSORT'} : 0;
 $SUBSORT     = defined($ENV{'M2H_SUBSORT'}) ? $ENV{'M2H_SUBSORT'} : 0;
 $AUTHSORT    = defined($ENV{'M2H_AUTHSORT'}) ? $ENV{'M2H_AUTHSORT'} : 0;
 $THREAD      = defined($ENV{'M2H_THREAD'}) ? $ENV{'M2H_THREAD'} : 1;
+$TNOSORT     = defined($ENV{'M2H_TSORT'}) ? !$ENV{'M2H_TSORT'} : 0;
 $TREVERSE    = defined($ENV{'M2H_TREVERSE'}) ? $ENV{'M2H_TREVERSE'} : 0;
+$TSUBSORT    = defined($ENV{'M2H_TSUBSORT'}) ? $ENV{'M2H_TSUBSORT'} : 0;
+$GzipFiles   = defined($ENV{'M2H_GZIPFILES'}) ? $ENV{'M2H_GZIPFILES'} : 0;
+$GzipLinks   = defined($ENV{'M2H_GZIPLINKS'}) ? $ENV{'M2H_GZIPLINKS'} : 0;
 
 if ($UNIX) {
     eval q/
@@ -299,12 +330,26 @@ if ($UNIX) {
 }
 
 $DecodeHeads = defined($ENV{'M2H_DECODEHEADS'}) ? $ENV{'M2H_DECODEHEADS'} : 0;
+$DoArchive   = defined($ENV{'M2H_ARCHIVE'}) ? $ENV{'M2H_ARCHIVE'} : 1;
 $DoFolRefs   = defined($ENV{'M2H_FOLREFS'}) ? $ENV{'M2H_FOLREFS'} : 1;
+$UsingLASTPG = defined($ENV{'M2H_USINGLASTPG'}) ? $ENV{'M2H_USINGLASTPG'} : 1;
 
 @OtherIdxs   = defined($ENV{'M2H_OTHERINDEXES'}) ?
 		    split(/:/, $ENV{'M2H_OTHERINDEXES'}) : ();
 @PerlINC     = defined($ENV{'M2H_PERLINC'}) ?
 		    split(/:/, $ENV{'M2H_PERLINC'}) : ();
+@DateFields  = defined($ENV{'M2H_DATEFIELDS'}) ?
+		    split(/:/, $ENV{'M2H_DATEFIELDS'}) : ();
+@FromFields  = defined($ENV{'M2H_FROMFIELDS'}) ?
+		    split(/:/, $ENV{'M2H_FROMFIELDS'}) : ();
+
+($TSliceNBefore, $TSliceNAfter) = defined($ENV{'M2H_TSLICE'}) ?
+		    split(/:/, $ENV{'M2H_TSLICE'}) : (0, 4);
+
+$SubArtRxp   = $ENV{'M2H_SUBJECTARTICLERXP'} ||
+	       q/^(the|a|an)\s+/;
+$SubReplyRxp = $ENV{'M2H_SUBJECTREPLYRXP'} ||
+	       q/^\s*(re|sv|fwd|fw)[\[\]\d]*[:>-]+\s*/;
 
 ##	Arrays for months and weekdays.  If empty, the default settings
 ##	in mhtime.pl are used.
@@ -330,10 +375,8 @@ $SUBJECTBEG	= '';		# Begin of subject group
 $SUBJECTEND	= '';		# End of subject group
 
 $TIDXLABEL	= '';		# Label for thread index
-$THEAD  	= '';		# Thread index header
-$TFOOT  	= '';		# Thread index footer
-$TLISTBEG	= '';		# Thread list open
-$TLISTEND	= '';		# Thread list close
+$THEAD  	= '';		# Thread index header (and list start)
+$TFOOT  	= '';		# Thread index footer (and list end)
 $TSINGLETXT	= '';		# Single/lone thread entry template
 $TTOPBEG	= '';		# Top of a thread begin template
 $TTOPEND	= '';		# Top of a thread end template
@@ -350,12 +393,16 @@ $TINDENTEND	= '';		# Thread indent close
 $TCONTBEG	= '';		# Thread continue open
 $TCONTEND	= '';		# Thread continue close
 
+$TSLICEBEG	= '';		# Start of thread slice
+$TSLICEEND	= '';		# End of thread slice
+
 $MSGFOOT	= '';		# Message footer
 $MSGHEAD	= '';		# Message header
 $TOPLINKS	= '';		# Message links at top of message
 $BOTLINKS	= '';		# Message links at bottom of message
 $SUBJECTHEADER	= '';		# Markup for message main subject line
 $HEADBODYSEP 	= '';		# Markup between mail header and body
+$MSGBODYEND 	= '';		# Markup at end of message data
 
 $FIELDSBEG	= '';		# Beginning markup for mail header
 $FIELDSEND	= '';		# End markup for mail header
@@ -400,6 +447,15 @@ $TNEXTPGLINKIA	= '';   	# Thread next page inactive link template
 $TPREVPGLINK	= '';   	# Thread previous page link template
 $TPREVPGLINKIA	= '';   	# Thread previous page inactive link template
 
+$FOLUPBEGIN	= '';		# Start of follow-ups for message page
+$FOLUPLITXT	= '';		# Markup for follow-up list entry
+$FOLUPEND	= '';		# End of follow-ups for message page
+$REFSBEGIN	= '';		# Start of refs for message page
+$REFSLITXT	= '';		# Markup for ref list entry
+$REFSEND	= '';		# End of refs for message page
+
+$MSGIDLINK 	= '';		# Markup for linking message-ids
+
 ##	The following associative array if for defining custom
 ##	resource variables
 %CustomRcVars	= ();
